#include <spu_intrinsics.h>
#include "sxm.h"
#include <stdio.h>


// LS data segment
vector unsigned int zero = { 0, 0, 0, 0 };
vector unsigned int one  = { 1, 1, 1, 1 };
vector unsigned int two  = { 2, 2, 2, 2 };
vector unsigned int a    = {  4000000000,  4000000000,  1500, 2000000 };
vector unsigned int b    = {  4000000000,  1000,  2000000000, 1500001 };
vector unsigned int ch;
vector unsigned int cl;
vector unsigned int dh   = {  3725290298, 931, 698, 698 };
vector unsigned int dl   = {  1983905792, 1385447424, 2112827392, 2114827392 };
vector unsigned int q;
vector unsigned int m;


int test_umul()
{
   int passed;

   sxm_umul( &ch, &cl, a, b );				// 4 multiplications

   if (   !spu_extract( spu_orx( spu_xor( cl, dl )), 0)   
       && !spu_extract( spu_orx( spu_xor( cl, dl )), 0)
      ) passed = 1;
   else passed = 0;

   return passed;
}


int test_div()
{
   int passed;

   vector unsigned int f ={ 1000, 200, 3000, 4000 };

   q = sxm_udivmod( dh, dl, b, zero );    		// 4 divisions
   m = sxm_udivmod( dh, spu_add(dl,f), b, one );    	// 4 modulos

   if (   !spu_extract( spu_orx( spu_xor( a, q )), 0)   
       && !spu_extract( spu_orx( spu_xor( m, f )), 0) 
      ) passed = 1;
   else passed = 0;

  return passed;
}

int test_mpow()
{
   int passed;
   vector unsigned int r;
   vector unsigned int s = { 96, 192, 84, 18 };

   q = (vector unsigned int){ 32, 33, 34, 35 };   	// 2^32 = 4294967296
   m = (vector unsigned int){ 100, 200, 100, 50 };

   r = sxm_umodpow( two, q, m );			// 4 modular exponentations

   if (  !spu_extract( spu_orx( spu_xor( r, s )), 0)  )
      passed = 1;
   else
      passed = 0;

   return passed;
}

int test_pow()
{
   int passed;
   vector unsigned int r;
   vector unsigned int s = { 1000000000, 1024, 4096, 100000000 };
   vector unsigned int b = { 10, 2, 2, 100 };   
   vector unsigned int e = { 9, 10, 12, 4 };

   r = sxm_upow( b, e );			// 4 exponentations

   if ( !spu_extract( spu_orx( spu_xor( r, s )), 0)  )
      passed = 1;
   else
      passed = 0;

   return passed;
}

int test_fdiv()
{
   int passed;
   vector float r;
   vector float s = { 1.0f, 0.5f, 0.25f, 0.125f };
   vector float t = { 0.2f, 0.3333333f, 0.5714285f, 1.0f };
   vector float b = { 5.0, 6.0f, 7.0f, 8.0f };   
   vector float e = { 1.0f, 2.0f, 4.0f, 8.0f };

   r = sxm_frecp( e );			// 4 reciprocals

   if (   spu_extract(r,0) >= 0.999999*spu_extract(s,0) && spu_extract(r,1) >= 0.999999*spu_extract(s,1) 
       && spu_extract(r,2) >= 0.999999*spu_extract(s,2) && spu_extract(r,3) >= 0.999999*spu_extract(s,3)
       && spu_extract(r,0) <= 1.000001*spu_extract(s,0) && spu_extract(r,1) <= 1.000001*spu_extract(s,1) 
       && spu_extract(r,2) <= 1.000001*spu_extract(s,2) && spu_extract(r,3) <= 1.000001*spu_extract(s,3)
      )
      passed = 1;
   else
      passed = 0;

   r = sxm_fdiv( e, b );		// 4 divisions

   if (   spu_extract(r,0) >= 0.999999*spu_extract(t,0) && spu_extract(r,1) >= 0.999999*spu_extract(t,1) 
       && spu_extract(r,2) >= 0.999999*spu_extract(t,2) && spu_extract(r,3) >= 0.999999*spu_extract(t,3)
       && spu_extract(r,0) <= 1.000001*spu_extract(t,0) && spu_extract(r,1) <= 1.000001*spu_extract(t,1) 
       && spu_extract(r,2) <= 1.000001*spu_extract(t,2) && spu_extract(r,3) <= 1.000001*spu_extract(t,3)
      )
      passed &= 1;
   else
      passed &= 0;

   return passed;
}


int main (unsigned long long spe_id, 
          unsigned long long argp, 
          unsigned long long envp) 
{
   printf("\nTesting libsxm.a -\n");

   if ( test_umul() ) printf("sxm_umul passed\n");
   else              printf("sxm_umul FAILED\n");

   if ( test_div() ) printf("sxm_udivmod passed\n");
   else              printf("sxm_udivmod FAILED\n");

   if ( test_mpow()) printf("sxm_umodpow passed\n");
   else              printf("sxm_umodpow FAILED\n");

   if ( test_pow() ) printf("sxm_upow passed\n");
   else              printf("sxm_upow FAILED\n");

   if ( test_fdiv() ) printf("sxm_fdiv passed\n");
   else               printf("sxm_fdiv FAILED\n");

   printf("test end.\n");
   return 0;
}

